@@ -47,6 +47,7 @@ struct mode_struct {
 	bool variable_units_enabled;
 	int keypad;
 	bool autocalc;
+	bool chain_mode;
 	bool complex_angle_form;
 };
 
@@ -136,7 +137,8 @@ enum {
 	SHORTCUT_TYPE_COPY_RESULT,
 	SHORTCUT_TYPE_SAVE_IMAGE,
 	SHORTCUT_TYPE_HELP,
-	SHORTCUT_TYPE_QUIT
+	SHORTCUT_TYPE_QUIT,
+	SHORTCUT_TYPE_CHAIN_MODE
 };
 
 struct keyboard_shortcut {
@@ -232,6 +234,7 @@ void on_tUnitSelectorCategories_selection_changed(GtkTreeSelection *treeselectio
 
 void execute_expression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0, std::string execute_str = std::string(), std::string str = std::string(), bool check_exrates = true);
 void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, std::string transformation = "", size_t stack_index = 0, bool register_moved = false, bool supress_dialog = false);
+void execute_from_file(std::string file_name);
 
 void set_rpn_mode(bool b);
 void calculateRPN(int op);
@@ -329,14 +332,17 @@ void edit_dataset(DataSet *ds = NULL, GtkWidget *win = NULL);
 
 void edit_names(ExpressionItem *item = NULL, const gchar *namestr = NULL, GtkWidget *win = NULL, bool is_dp = false, DataProperty *dp = NULL);
 
-gchar *font_name_to_css(const char *font_name);
+gchar *font_name_to_css(const char *font_name, const char *w = "*");
 
 void reload_history(gint from_index = -1);
+void set_status_bottom_border_visible(bool);
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+void update_menu_base();
+
 void memory_recall();
 void memory_store();
 void memory_add();
@@ -509,6 +515,7 @@ void on_menu_item_new_function_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_dataset_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_unit_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_autocalc_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_chain_mode_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_rpn_mode_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_rpn_syntax_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_limit_implicit_multiplication_activate(GtkMenuItem *w, gpointer user_data);
@@ -673,8 +680,10 @@ gboolean on_main_window_focus_in_event(GtkWidget *w, GdkEventFocus *e, gpointer
 
 void on_button_registerup_clicked(GtkButton *button, gpointer user_data);
 void on_button_registerdown_clicked(GtkButton *button, gpointer user_data);
+void on_button_registerswap_clicked(GtkButton *button, gpointer user_data);
 void on_button_editregister_clicked(GtkButton *button, gpointer user_data);
 void on_button_deleteregister_clicked(GtkButton *button, gpointer user_data);
+void on_button_copyregister_clicked(GtkButton *button, gpointer user_data);
 void on_button_clearstack_clicked(GtkButton *button, gpointer user_data);
 void on_stackview_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
 void on_stackview_item_edited(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer user_data);
